package me.icymint.libra.sas.scanner;

import java.util.concurrent.LinkedBlockingQueue;

import me.icymint.libra.sas.config.SasConfig;
import me.icymint.libra.sas.exception.SasAbortException;
import me.icymint.libra.sas.exception.SasCacheFileLockedException;
import me.icymint.libra.sas.exception.SasInterruptException;
import me.icymint.libra.sas.exception.SasNotRunException;
import me.icymint.libra.sas.exception.SasRunErrorException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractScanner<V, T extends SasConfig> implements
		Scanner<V, T> {
	protected final Logger logger = LoggerFactory.getLogger(getClass());
	private final T config;
	protected final LinkedBlockingQueue<V> v = new LinkedBlockingQueue<V>();

	public AbstractScanner(T config) {
		this.config = config;
	}

	@Override
	public void execute() throws SasRunErrorException, SasInterruptException,
			SasAbortException, SasNotRunException, SasCacheFileLockedException {
		initializeScanner(config);
		while (true) {
			scan(config);
			try {
				Thread.sleep(10);
			} catch (InterruptedException e) {
			}
			if (config.isFinished())
				break;
		}
		scan(config);
		finallizeScanner(config);
	}

	@Override
	public void finallizeScanner(T config) throws SasRunErrorException {
		if (config.isError())
			throw new SasRunErrorException(config.getErrorFlag());
	}

	public T getConfig() {
		return config;
	}

	/**
	 * 初始化文件夹。
	 * 
	 * @throws SasNotRunException
	 * @throws SasCacheFileLockedException
	 */
	@Override
	public void initializeScanner(T config) throws SasNotRunException,
			SasCacheFileLockedException {
		/**
		 * 目录结构的初始化。
		 */
		if (config.isFlagExist())
			throw new SasCacheFileLockedException(config.getCacheRoot());
		else {
			config.getCacheRoot().mkdirs();
			config.getFlagRoot().mkdirs();
			config.getLogRoot().mkdirs();
			config.createFlag();
		}
		/**
		 * SAS执行文件的初始化。
		 */
		config.output(this.sas());
	}

	/**
	 * 用于关闭
	 */
	public void shutdown() {
	}

	@Override
	public V take() {
		if (config.isFinished() && v.size() == 0)
			return null;
		else
			try {
				return v.take();
			} catch (InterruptedException e) {
				logger.warn("结果获取被中断");
				return this.take();
			}
	}
}
